/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_GL_BOXARRAY_H
#define X3DTK_GL_BOXARRAY_H

#include "GL_SceneGraphTypes.h"

#include <vector>

namespace X3DTK {
namespace GL {

/*! \brief Class declared in <b><X3DTK/GL/scenegraph.h></b> providing a singleton manager
 *  of the vertex arrays necessary to draw a box. 
 *  
 *  This class is used by Box to avoid memory duplication and provides the arrays required by the OpenGL command
 *  drawElements().
 *
 *  \ingroup GLSceneGraph
 */

class BoxDrawArray
{
public:
  /// Gets the CylinderDrawArray of section section.
  static BoxDrawArray *getInstance();
  /// Removes the CylinderDrawArray of section section.
  void removeInstance();

  /// Gets the size of the Box, it means the number of vertices displayed.
  unsigned int getBoxSize() const;
  /// Gets the address of the first element of the vertex array.
  const void *getBoxVertexArrayAddress() const;
  /// Gets the address of the first index.
  const unsigned int *getBoxIndexArrayAddress() const;

private:
  /// Constructor.
  BoxDrawArray();
  
  static int _count;
  static BoxDrawArray *_ref;
  
  std::vector<N3F_V3F> _boxVertexArray;
  std::vector<unsigned int> _boxIndexArray;
};

}
}

#endif
