/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            ClonerStateVariables.h                         //
////////////////////////////////////////////////////////////////////////////////

#ifndef CLONERGLOBALVARIABLES_H
#define CLONERGLOBALVARIABLES_H

#include "StateVariables.h"

namespace X3DTK {

/// StateVariables for the Cloner module.

class ClonerStateVariables : public StateVariables
{
public:
  /// Constructor.
  ClonerStateVariables();
  /// Destructor.
  virtual ~ClonerStateVariables();

  /// Initializes the root of the cloned graph.
  void initRoot();
  /// Initializes the traversal.
  virtual void init();
  /// Finishes the traversal.
  virtual void finish();
  
  /// Adds the couple of nodes.
  void addCoupleNode(SFNode N, SFNode NN);
  /// Pushes the node.
  void pushNode(SFNode N);
  /// Pops the node.
  void popNode();
    
  /// Gets the GLScene.
  inline SFNode getRoot() const {return _root;};
  /// Gets the top of the stack.
  inline SFNode getTop() const {return (!_nodeStack.empty() ? _nodeStack.front() : 0);};
  /// Gets the GL::SFNode related with N. Returns 0 if N not found.
  SFNode getNode(SFNode N) const;
  
private: 
  SFNode _root;
  MFNode _nodeStack;
  std::map<SFNode, SFNode> _nodeCoupleMap;
};

}

#endif
