/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_MESH_EDGESDRAWERSTATEVARIABLES_H
#define X3DTK_MESH_EDGESDRAWERSTATEVARIABLES_H

#include "MESH_Transform.h"
#include "MESH_SceneGraphTypes.h"

#include <list>
#include <vector>

namespace X3DTK {
namespace MESH {

/*! \brief Class declared in <b><X3DTK/MESH/edgesdrawer.h></b> providing the StateVariables
 *  of the MESH::TemplateEdgesDrawer processor.
 *
 *  \ingroup MESH
 */
 
template<class MData, class VData, class EData, class FData, bool RW>
class TemplateEdgesDrawerStateVariables : public StateVariables
{
public:
  // Constructor.
  TemplateEdgesDrawerStateVariables();

  // Initializes the traversal.
  void init();
  // Finishes the traversal.
  void finish();

  // Pushes the transformation node.
  void pushTransform(const SFMatrix34f &matrix);
  /// Gets the current Transform node.
  SFMatrix34f getCurrentTransform();
  // Pops the transformation node.
  void popTransform();

  // Adds the couple of nodes that have been met.
  void addCouple(X3DNode *N, const SFMatrix34f &matrix);
  // Returns TRUE if the couple of nodes have already been met.
  bool getCouple(X3DNode *N, const SFMatrix34f &matrix) const;
  
  // Saves current matrix state and returns index in saved vector.
  // Call storeMesh() before calling ths function
  int getCurrentMeshId();
  // Saves mesh and associated current matrix.
  void storeVertex(TemplateVertex<MData, VData, EData, FData, RW> *vertex);
  
  // Get ith Vertex of the model.
  inline TemplateVertex<MData, VData, EData, FData, RW> *getVertex(int i) const;
  // Gets stored transformation matrix associated with a given Mesh.
  inline const SFMatrix34f &getMatrix(int i) const;

  // Whether or not primitive IDs are pushed.
  inline void setSelectionMode(bool selectionMode);
  inline bool getSelectionMode() const;
    
private: 
  std::list<SFMatrix34f> _matrixStack;
  std::vector<SFMatrix34f> _matrixVector;
  std::list<std::pair<X3DNode *, SFMatrix34f> > _coupleList;
  std::vector<TemplateVertex<MData, VData, EData, FData, RW> *> _vertexVector;
  bool _selectionMode;
};

}
}

#include "MESH_EdgesDrawerStateVariables.inl"

#endif
