/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_MESH_NORMALSDRAWERSTATEVARIABLES_H
#define X3DTK_MESH_NORMALSDRAWERSTATEVARIABLES_H

#include "MESH_SceneGraphTypes.h"
#include "MESH_Mesh.h"
#include "StateVariables.h"

#include <list>

namespace X3DTK {
namespace MESH {

/*! \brief Class declared in <b><X3DTK/MESH/normalsdrawer.h></b> providing the StateVariables
 *  of the MESH::TemplateNormalsDrawer processor.
 *
 *  \ingroup MESH
 */
 
template<class MData, class VData, class EData, class FData, bool RW>
class TemplateNormalsDrawerStateVariables : public StateVariables
{
public:
  /// Constructor.
  TemplateNormalsDrawerStateVariables();

  // Initializes the traversal.
  void init();
  // Finishes the traversal.
  void finish();

  /// Sets the amplitude.
  void setAmplitude(float amplitude);
  /// Gets the amplitude.
  inline float getAmplitude() const;
  
  // Pushes the transformation matrix.
  void pushMatrix(const SFMatrix34f &transformation);
  // Pops the transformation matrix.
  void popMatrix();

private: 
  std::list<SFMatrix34f> _matrixStack;
  float _amplitude;
};

}
}

#include "MESH_NormalsDrawerStateVariables.inl"

#endif
