/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_MESH_NORMALSFLIPPER_H
#define X3DTK_MESH_NORMALSFLIPPER_H

#include "MESH_NormalsFlipperStateVariables.h"
#include "X3DOnePassProcessor.h"
#include "MESH_SceneGraphTypes.h"
#include "DFSGraphTraversal.h"
#include "MESH_NormalsFlipperCoreVisitor.h"
#include "MESH_Mesh.h"
#include "MESH_Scene.h"
#include "MESH_MeshData.h"

namespace X3DTK {
namespace MESH {

/*! \brief Class declared in <b><X3DTK/MESH/normalsflipper.h></b> providing a processor that 
 *  makes the orientation of the faces of a MESH scene graph coherent.
 *
 *  For each connected component, the algorithm counts the number of relative 
 *  counterclockwise faces against the number of not counterclockwise faces. The minority 
 *  is inversed so that all the faces are coherent. 
 * 
 *  \ingroup MESH
 */

template<class MData, class VData, class EData, class FData, bool RW>
class TemplateNormalsFlipper : public X3DOnePassProcessor
{
public:
  /// Constructor.
  TemplateNormalsFlipper();
  /// Destructor.
  virtual ~TemplateNormalsFlipper();
  
  /// Flips the normals of the scene.
  void flip(Scene *S);
  /// Flips the normals.
  void flip(X3DNode *N);
  /// Sets the selected nodes to update. If empty, then all the graph is updated.
  void setSelectedNodes(const MFNode &selectedNodes);
};

/// Default TemplateNormalsFlipper.
typedef TemplateNormalsFlipper<MeshData, VertexData, EdgeData, FaceData, true> NormalsFlipper;

}
}

#include "MESH_NormalsFlipper.inl"

#endif

