/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3DCOMPONENT_H
#define X3DTK_X3DCOMPONENT_H

#include "X3DTypes.h"
#include "SFComponent.h"

namespace X3DTK {

/*! \brief Abstract Class declared in <b><X3DTK/kernel.h></b> defining the concept of component.
 *
 *  \ingroup processor
 */

class X3DComponent
{
public:
  /// Constructor.
  X3DComponent();
  /// Destructor.
  virtual ~X3DComponent() = 0;
  
  /// Returns the component.
  inline SFComponent *getComponent() const {return component;};
  /// Returns the name of the component.
  inline SFString getName() const {return component->getName();};
  
  /// Adds one actor.
  void addOneActor();
  /// Removes one actor.
  void removeOneActor();
  
  /// Returns the number of actors.
  inline unsigned int getActorNumber() const {return _actorNumber;};
  
protected:
  /// Defines the name of the component.
  void defineComponentName(const SFString &name, const SFString &sceneGraph);
  /// Pointer to the singleton component.
  SFComponent *component;

private:  
  unsigned int _actorNumber;  
};

}

#endif
