/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3D_APPEARANCE_H
#define X3DTK_X3D_APPEARANCE_H

#include "X3DTypes.h"
#include "X3D_X3DAppearanceNode.h"

namespace X3DTK {
namespace X3D {

class X3DMaterialNode;
class X3DTextureNode;
class X3DTextureTransformNode;
class LineProperties;
class FillProperties;

/*! \brief Class declared in <b><X3DTK/X3D/scenegraph.h></b>, part of the <a href="../../doc/X3DSpecs/Part01/components/shape.html">shape component</a> defining an Appearance
 *  (<a href="../../doc/X3DSpecs/Part01/components/shape.html#Appearance">
 *  documentation</a>).
 *
 *  \ingroup X3DSceneGraph
 */

class Appearance : public X3DAppearanceNode
{
public:
  /// Constructor.
  Appearance();
  /// Destructor.
  virtual ~Appearance();
  
  /// Gets the Material attribute.  
  inline X3DMaterialNode *getMaterial() const {return _material;};
  /// Gets the Texture attribute.  
  inline X3DTextureNode *getTexture() const {return _texture;};
  /// Gets the TextureTransform attribute.  
  inline X3DTextureTransformNode *getTextureTransform() const {return _textureTransform;};
  /// Gets the LineProperties attribute.  
  inline LineProperties *getLineProperties() const {return _lineProperties;};
  /// Gets the FillProperties attribute.  
  inline FillProperties *getFillProperties() const {return _fillProperties;};

private:  
  /// Material node.
  X3DMaterialNode *_material;
  /// Texture node.
  X3DTextureNode *_texture;
  /// Texture transform node.
  X3DTextureTransformNode *_textureTransform;
  /// Line properties.
  LineProperties *_lineProperties;
  /// Fill properties.
  FillProperties *_fillProperties;
};

}
}

#endif
