/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3D_FILEVALIDATOR_H
#define X3DTK_X3D_FILEVALIDATOR_H

#include "X3DOnePassProcessor.h"
#include "X3D_FileValidatorStateVariables.h"
#include "X3D_X3DNode.h"

namespace X3DTK {
namespace X3D {

/*! \brief Class declared in <b><X3DTK/X3D/filevalidator.h></b> providing a processor
 *  that validates an X3D file, and particularly the validity of the X3DTK::X3D::IndexedFaceSet
 *  nodes. 
 *
 *  X3DTK::X3D::IndexedFaceSet have index arrays which can point to nothing. The processor
 *  detects which fields of the X3DTK::X3D::IndexedFaceSet are bad and prints them to
 *  the standard output. This validation cannot be done in the xml validation phase.
 *
 *  \ingroup X3D
 */

class FileValidator : public X3DOnePassProcessor
{
public:
  /// Constructor.
  FileValidator();
  /// Destructor.
  virtual ~FileValidator();
  
  /// Validates the file.
  virtual bool validate(SFNode N) const;
};

}
}

#endif
