/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3D_NORMAL_H
#define X3DTK_X3D_NORMAL_H

#include "X3DTypes.h"
#include "X3D_X3DNormalNode.h"

namespace X3DTK {
namespace X3D {

/*! \brief Class declared in <b><X3DTK/X3D/scenegraph.h></b>, part of the <a href="../../doc/X3DSpecs/Part01/components/rendering.html">
 *  Rendering component</a> defining a Normal
 *  (<a href="../../doc/X3DSpecs/Part01/components/rendering.html#Normal">
 *  documentation</a>).
 *
 *  \ingroup X3DSceneGraph
 */

class Normal : public X3DNormalNode
{
public:
  /// Constructor.
  Normal();
  /// Destructor.
  virtual ~Normal();
  
  /// Sets the Vector attribute.
  void setVector(const MFVec3f &vector);

  /// Gets the Vector attribute.
  inline MFVec3f &getVector() {return _vector;};
  /// Gets the Vector attribute.
  inline const MFVec3f &getVector() const {return _vector;};
  
private:  
  /// vector attribute.
  MFVec3f _vector;  
};

}
}

#endif
