/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3D_X3DTEXTURETRANSFORM2DNODE_H
#define X3DTK_X3D_X3DTEXTURETRANSFORM2DNODE_H

#include "X3DTypes.h"
#include "X3D_X3DTextureTransformNode.h"

namespace X3DTK {
namespace X3D {

/*! \brief Class declared in <b><X3DTK/X3D/scenegraph.h></b>, part of the <a href="../../doc/X3DSpecs/Part01/components/texturing.html">
 *  Texturing component</a> defining a X3DTextureTransform2DNode
 *  (<a href="../../doc/X3DSpecs/Part01/components/texturing.html#X3DTextureTransform2DNode">
 *  documentation</a>).
 *
 *  \ingroup X3DSceneGraph
 */

class X3DTextureTransform2DNode : public X3DTextureTransformNode
{
public:  
  /// Constructor.
  X3DTextureTransform2DNode();
  /// Destructor.
  virtual ~X3DTextureTransform2DNode();
  
  /// Sets the center of the transformation.
  void setCenter(const SFVec2f &center);
  /// Sets the rotation of the transformation.
  void setRotation(const SFFloat &rotation);
  /// Sets the scale of the transformation.
  void setScale(const SFVec2f &scale);
  /// Sets the translation of the transformation.
  void setTranslation(const SFVec2f &translation);

  /// Gets the center of the transformation.
  inline const SFVec2f &getCenter() const {return _center;};
  /// Gets the rotation of the transformation.
  inline const SFFloat &getRotation() const {return _rotation;};
  /// Gets the scale of the transformation.
  inline const SFVec2f &getScale() const {return _scale;};
  /// Gets the translation of the transformation.
  inline const SFVec2f &getTranslation() const {return _translation;};

private:
  /// Center attribute.
  SFVec2f _center;
  /// Rotation attribute.
  SFFloat _rotation;
  /// Scale attribute.
  SFVec2f _scale;
  /// Translation attribute.
  SFVec2f _translation;
};

}
}

#endif
